from __future__ import absolute_import

from .celery import celery

import subprocess
import pymongo 
import os
import datetime


if not hasattr(pymongo, 'MongoClient'):
    pymongo.MongoClient = pymongo.Connection

@celery.task(name='jobs.tasks.compute_sas')
def compute_sas(image_file_name, mesh_file_name, measures_file_name):
    binaryname = celery.conf.SAS_BIN
    process = subprocess.Popen([binaryname, 
                                image_file_name, 
                                "-C", 
                                "-O "+mesh_file_name, 
                                "-o "+measures_file_name], 
                               stdout=subprocess.PIPE, 
                               stderr=subprocess.STDOUT) 
    output = process.communicate()[0] 
    ret = process.wait()
    if (ret != 0):
        raise Exception("SAS BIN ERROR "+ output)
    return (image_file_name, mesh_file_name, measures_file_name)


@celery.task(name='jobs.tasks.clean')
def clean(clean_all=False):
    client = pymongo.connection.MongoClient()
    db = client.sas_web
    
    threshold_time = datetime.datetime.utcnow() - celery.conf.CLEANING_OLD_AGE
    condition = {} if clean_all else {'creation': {'$lt':threshold_time}} 

    
    old_files = db.files.find(condition)
    n_old_files = old_files.count()

    for old_file in old_files:
        for filename in ['upload_filename','sas_filename','measures_filename']:
            if os.path.isfile(old_file[filename]):
                os.remove(old_file[filename])

    db.files.remove(condition)
    client.close()
    return 'removed: %d' %n_old_files
