import os

from flask import render_template, request, url_for, jsonify, abort, g
from sas import sas
from werkzeug import secure_filename

import pymongo

import datetime
import uuid

from utils import *
from jobs import tasks



@sas.before_request
def before_request():
    g.db = connect_db()
    
@sas.teardown_request
def teardown_request(exception):
    g.db.connection.close()
        
@sas.route('/')
def index():
    return render_template('index.html')

def is_there_result(fileid):
    vtk_file_exists = os.path.isfile(get_sas_file(fileid))
    measures_file_exists = os.path.isfile(get_measusers_file(fileid))
    return vtk_file_exists and measures_file_exists

@sas.route('/results')
def results():
    template_info = {}
    template_info['task_status'] = None
    fileid = template_info['fileid'] = request.args.get('fileid', '')
    template_info['sas_file'] = get_sas_file(fileid)
    template_info['upload_file'] = get_upload_file(fileid)


    # Render result if files exist
    if is_there_result(fileid):
        with sas.open_resource(get_measusers_file(fileid)) as f:
            content = f.readlines()
            template_info['measures'] = [(l.split(':')[0].strip(), 
                                          l.split(':')[1].replace(';','').strip()) 
                                         for l in content]
        return render_template('results.html', template_info=template_info)

    # Check that the fileid is in the db
    file_info = g.db.files.find_one({'_id':fileid})
    if file_info is None:
        template_info['task_status'] =  'FILEID NOT FOUND'
        return render_template('results-fail.html', template_info=template_info)

    job_id = file_info['job_id']
    task_result = tasks.celery.AsyncResult(job_id)

    task_status = template_info['task_status'] = task_result.status
    
    if task_status == 'STARTED':
        return render_template('results-started.html', template_info=template_info)
    if task_status == 'PENDING':
        return render_template('results-pending.html', template_info=template_info)
    if task_status == 'FAILURE':
        return render_template('results-fail.html', template_info=template_info)
    
@sas.route('/examples')
def examples():
    return render_template('examples.html')

@sas.route('/+upload', methods=['GET', 'POST'])
def upload():
    if request.method == 'POST':
        data_file = request.files.get('data_file')
        if data_file and allowed_file(data_file.filename):
            filename = secure_filename(data_file.filename)
            fileid = uuid.uuid4().hex
            extension = filename.rsplit('.', 1)[1].lower()
            
            new_filename = get_upload_file(fileid, extension)
            sas_filename = get_sas_file(fileid)
            measures_filename = get_measusers_file(fileid)

            try:
                obj = None
                data_file.save(new_filename)
                result = tasks.compute_sas.delay(new_filename,
                                                 sas_filename,
                                                 measures_filename)
                obj = g.db.files.insert({'_id':fileid,
                                         'original_filename':filename,
                                         'upload_filename':new_filename,
                                         'sas_filename':sas_filename,
                                         'measures_filename':measures_filename,
                                         'extension':extension,
                                         'job_id':result.id,
                                         'creation':datetime.datetime.utcnow()}
                                        )
                
            except Exception, e:
                if obj is not None: g.db.files.remove(obj)
                return jsonify(name=filename, fileid="", error=True, msg="Imposible to store the file")
                #abort(500)
                
            return jsonify(name=filename, fileid=fileid, error=False)
        #abort(415) # Unsuported MimeType
        return jsonify(name="", fileid="", error=True, msg="Unsupported image extension")
    if request.method == 'GET':
        return "GET"
            
